/**
 *	@file	test.cpp
 *	@brief
 *	@author	tenk*
 *	@date	2010-??
 *	@note
 *		fname.h ̃eXg.
 */

#define USE_UNITST

//#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include "../fname.h"

#define USE_UNITST_STRSTREAM
#include "unitst.h"


// ===========================================================================
#ifdef UNICODE
#define T(x)		L##x
#else
#define T(x)		x
#endif

#if defined _WIN32 || defined _DOS || defined _MSDOS
#define IF_W32	if (1)
#else
#define IF_W32	if (0)
#endif

UNITST_TEST(fname) {
  #if defined _WIN32 || defined _DOS || defined _MSDOS
	const FNAME_CHAR*	test1 = T("C:\\Dummy\\File.txt");
	const FNAME_CHAR*	test2 = T("c:\\Dummy/FiLE.txt");
  #else
	const FNAME_CHAR*	test1 = T("C:/Dummy/File.txt");
	const FNAME_CHAR*	test2 = T("C:/Dummy/File.txt");
  #endif
	FNAME_CHAR			buf[FNAME_MAX_PATH];

	IF_W32 UNITST_CHECK( fname_cmp(T("C:\\Dummy\\File.txt"), T("d:")) < 0 );
	else   UNITST_CHECK( fname_cmp(T("C:/Dummy/File.txt"), T("d:")) < 0 );
	UNITST_CHECK( fname_cmp(test1, T("B:")) > 0 );
	UNITST_CHECK( fname_cmp(test1, test2) == 0 );

	UNITST_CHECK( fname_n_cmp(test1, T("d:"), 2) < 0 );
	IF_W32 UNITST_CHECK( fname_n_cmp(test1, T("b:\\"), 3) > 0 );
	else   UNITST_CHECK( fname_n_cmp(test1, T("B:/"), 3) > 0 );
	IF_W32 UNITST_CHECK( fname_n_cmp(test1, T("c:\\De"), 4) == 0 );
	IF_W32 UNITST_CHECK( fname_n_cmp(test1, T("c:\\De"), 5) > 0 );
	UNITST_CHECK( fname_n_cmp(test1, test2, 0xffff) == 0 );

	IF_W32 UNITST_CHECK( fname_equLong(test1, T("C:\\Dummy")) != 0 );
	else   UNITST_CHECK( fname_equLong(test1, T("C:/Dummy")) != 0 );
	IF_W32 UNITST_CHECK( fname_equLong(test1, T("c:/DuMmy")) != 0 );
	else   UNITST_CHECK( fname_equLong(test1, T("c:/DuMmy")) == 0 );
	UNITST_CHECK( fname_equLong(test1, T("C:/Bs")) == 0 );

	IF_W32 UNITST_CHECK( fname_isAbs(T("c:\\test1.txt")) != 0);
	UNITST_CHECK( fname_isAbs(T("c:/test1.txt")) != 0);
	IF_W32 UNITST_CHECK( fname_isAbs(T("\\test1.txt")) != 0);
	UNITST_CHECK( fname_isAbs(T("/test1.txt")) != 0);
	UNITST_CHECK( fname_isAbs(T("c:test1.txt")) == 0 );
	UNITST_CHECK( fname_isAbs(T("test1.txt")) == 0 );

	UNITST_CHECK( fname_haveDrive(T("c:test1.txt")) != 0);
	UNITST_CHECK( fname_haveDrive(T("test1.txt")) == 0 );

	UNITST_EQ( fname_adjust_size(T("c:/test.c"), 20), 9 );
	UNITST_EQ( fname_adjust_size(T("c:/test.c"), 10), 9 );
	UNITST_EQ( fname_adjust_size(T("c:/test.c"),  9), 8 );
  #if defined _WIN32 && !defined UNICODE
	UNITST_EQ( fname_adjust_size(T("c:eXg.c"), 8), 6 );
  #endif

	UNITST_EQ_TSTR( fname_cpy(buf, FNAME_MAX_PATH, T("abc.txt") ), T("abc.txt") );
	UNITST_EQ_TSTR( fname_cpy(buf, 4		 , T(""))		 , T("")		);
	UNITST_EQ_TSTR( fname_cpy(buf, 8		 , T("abc.txt") ), T("abc.txt") );
	UNITST_EQ_TSTR( fname_cpy(buf, 7		 , T("abc.txt") ), T("abc.tx")	);
  #if defined _WIN32 && !defined UNICODE
	UNITST_EQ_TSTR( fname_cpy(buf, 8		 , T("abc.g")), T("abc.g")	);
  #endif
	UNITST_EQ_TSTR( fname_cpy(buf, 1		 , T("abc.g")), T("")		);
	UNITST_EQ_TSTR( fname_cpy(buf, 5		 , T("abc.txt") ), T("abc.")	);

  #ifndef UNICODE
	UNITST_EQ_TSTR( fname_cat(buf, 7		 , T("g") )	 , T("abc.g")	);
	UNITST_EQ_TSTR( fname_cat(buf, 20		 , T("q") )	 , T("abc.gq") );
	UNITST_EQ_TSTR( fname_cat(buf, 20		 , T("")	 )	 , T("abc.gq") );
	UNITST_EQ_TSTR( fname_cat(buf, 7		 , T("") )	 , T("abc.g") );
  #endif

	UNITST_EQ_TSTR( fname_baseName(T("abc.text")),			   T("abc.text") );
	UNITST_EQ_TSTR( fname_baseName(T("a:abc.text")),		   T("abc.text") );
	IF_W32 UNITST_EQ_TSTR( fname_baseName(T("c:\\abc.text")),		   T("abc.text") );
	UNITST_EQ_TSTR( fname_baseName(T("c:/abc.text")),		   T("abc.text") );
	IF_W32 UNITST_EQ_TSTR( fname_baseName(T("c:\\basesss/abc.text")), T("abc.text") );
	UNITST_EQ_TSTR( fname_baseName(T("c:/basesss/abc.text")), T("abc.text") );
	UNITST_EQ_TSTR( fname_baseName(T("/basesss/abc.text")),    T("abc.text") );

	UNITST_EQ_TSTR( fname_ext(T("/basesss/abc.ext")),	 T("ext") );
	UNITST_EQ_TSTR( fname_ext(T("/basesss/abc")    ),	 T("") );
	UNITST_EQ_TSTR( fname_ext(T("/basesss/abc.")   ),	 T("") );

	UNITST_EQ_TSTR( fname_skipDrive(T("abc.text")),			  T("abc.text") );
	UNITST_EQ_TSTR( fname_skipDrive(T("a:abc.text")), 		  T("abc.text") );
	IF_W32 UNITST_EQ_TSTR( fname_skipDrive(T("c:\\abc.text")),		  T("\\abc.text") );
	UNITST_EQ_TSTR( fname_skipDrive(T("c:/abc.text")),		  T("/abc.text") );
	IF_W32 UNITST_EQ_TSTR( fname_skipDrive(T("c:\\basesss\\abc.text")), T("\\basesss\\abc.text") );
	UNITST_EQ_TSTR( fname_skipDrive(T("c:/basesss/abc.text")), T("/basesss/abc.text") );
	UNITST_EQ_TSTR( fname_skipDrive(T("/basesss/abc.text")),	  T("/basesss/abc.text") );

	UNITST_EQ_TSTR( fname_skipDriveRoot(T("abc.text")),			  T("abc.text") );
	UNITST_EQ_TSTR( fname_skipDriveRoot(T("a:abc.text")), 		  T("abc.text") );
	IF_W32 UNITST_EQ_TSTR( fname_skipDriveRoot(T("c:\\abc.text")),		  T("abc.text") );
	UNITST_EQ_TSTR( fname_skipDriveRoot(T("c:/abc.text")),		  T("abc.text") );
	IF_W32 UNITST_EQ_TSTR( fname_skipDriveRoot(T("c:\\basesss/abc.text")), T("basesss/abc.text") );
	UNITST_EQ_TSTR( fname_skipDriveRoot(T("c:/basesss/abc.text")), T("basesss/abc.text") );
	UNITST_EQ_TSTR( fname_skipDriveRoot(T("/basesss/abc.text")),	  T("basesss/abc.text") );

	UNITST_EQ_TSTR( fname_delExt(fname_cpy(buf, 100, T("/basesss/abc.ext"))), T("/basesss/abc") );
	UNITST_EQ_TSTR( fname_delExt(fname_cpy(buf, 100, T("abc.ext")		  )), T("abc") );
	UNITST_EQ_TSTR( fname_delExt(fname_cpy(buf, 100, T("abc.")			  )), T("abc") );
	UNITST_EQ_TSTR( fname_delExt(fname_cpy(buf, 100, T("abc")			  )), T("abc") );

	UNITST_EQ_TSTR( fname_getNoExt(buf, 100, T("c:abcd/abc.defghijk")), T("c:abcd/abc") );
	UNITST_EQ_TSTR( fname_getNoExt(buf, 100, T("abc.def")			 ), T("abc") );
	UNITST_EQ_TSTR( fname_getNoExt(buf, 100, T("abc.")				 ), T("abc") );
	UNITST_EQ_TSTR( fname_getNoExt(buf, 100, T("abc")				 ), T("abc") );

	UNITST_EQ_TSTR( fname_getBaseNameNoExt(buf, 100, T("c:abcd/abc.defghijk")), T("abc") );
	UNITST_EQ_TSTR( fname_getBaseNameNoExt(buf, 100, T("c:dir/abc.")		 ), T("abc") );
	UNITST_EQ_TSTR( fname_getBaseNameNoExt(buf, 100, T("c:dir/abc") 		 ), T("abc") );

	UNITST_EQ_TSTR( fname_setExt(buf, 100, T("c:abcd/abc.def"), T("txt")), T("c:abcd/abc.txt") );
	UNITST_EQ_TSTR( fname_setExt(buf, 100, T("c:abcd/abc."	 ), T("txt")), T("c:abcd/abc.txt") );
	UNITST_EQ_TSTR( fname_setExt(buf, 100, T("c:abcd/abc"	 ), T("txt")), T("c:abcd/abc.txt") );
	UNITST_EQ_TSTR( fname_setExt(buf, 100, T("c:abcd/abc.def"), T(""))	 , T("c:abcd/abc") );
	UNITST_EQ_TSTR( fname_setExt(buf, 100, T("c:abcd/abc."	 ), T(""))	 , T("c:abcd/abc") );
	UNITST_EQ_TSTR( fname_setExt(buf, 100, T("c:abcd/abc"	 ), T(""))	 , T("c:abcd/abc") );

	UNITST_EQ_TSTR( fname_setDefaultExt(buf, 100, T("c:abcd/abc.def"), T("txt")), T("c:abcd/abc.def") );
	UNITST_EQ_TSTR( fname_setDefaultExt(buf, 100, T("c:abcd/abc."	), T("txt")), T("c:abcd/abc.txt") );
	UNITST_EQ_TSTR( fname_setDefaultExt(buf, 100, T("c:abcd/abc"	), T("txt")), T("c:abcd/abc.txt") );
	UNITST_EQ_TSTR( fname_setDefaultExt(buf, 100, T("c:abcd/abc.def"), T(""))	, T("c:abcd/abc.def") );
	UNITST_EQ_TSTR( fname_setDefaultExt(buf, 100, T("c:abcd/abc."	), T(""))	, T("c:abcd/abc") );
	UNITST_EQ_TSTR( fname_setDefaultExt(buf, 100, T("c:abcd/abc"	), T(""))	, T("c:abcd/abc") );

	UNITST_CHECK( fname_checkLastSep( T("abx") ) == 0 );
  #ifndef UNUSE_SJIS
	UNITST_CHECK( fname_checkLastSep( T("\") ) == 0 );
  #endif
	UNITST_CHECK( fname_checkLastSep( T("") ) == 0 );
	UNITST_EQ_TSTR( fname_checkLastSep( T("abx/") ), T("/") );
	IF_W32 UNITST_EQ_TSTR( fname_checkLastSep( T("abx\\") ), T("\\") );
	UNITST_EQ_TSTR( fname_checkLastSep( T("abx/") ), T("/") );
	UNITST_EQ_TSTR( fname_checkLastSep( T("/") ), T("/") );
	IF_W32 UNITST_EQ_TSTR( fname_checkLastSep( T("\\") ), T("\\") );

	IF_W32 UNITST_EQ_TSTR( fname_join(buf, 100, T("c:abcd/def"), T("test.txt")), T("c:abcd/def\\test.txt") );
	else   UNITST_EQ_TSTR( fname_join(buf, 100, T("c:abcd/def"), T("test.txt")), T("c:abcd/def/test.txt") );

	IF_W32 UNITST_EQ_TSTR( fname_join(buf, 100, T("c:abcd/def"), T("ghi/test.txt")), T("c:abcd/def\\ghi/test.txt") );
	else   UNITST_EQ_TSTR( fname_join(buf, 100, T("c:abcd/def"), T("ghi/test.txt")), T("c:abcd/def/ghi/test.txt") );

	UNITST_EQ_TSTR( fname_join(buf, 100, T("c:abcd/def/"), T("test.txt")), T("c:abcd/def/test.txt") );
	UNITST_EQ_TSTR( fname_join(buf, 100, T("c:abcd/def/"), T("ghi/test.txt")), T("c:abcd/def/ghi/test.txt") );

	UNITST_EQ_TSTR( fname_getDir(buf, 100, T("c:abcd/abc.def")) 	 , T("c:abcd") );
	UNITST_EQ_TSTR( fname_getDir(buf, 100, T("/abc/def/hi/jk.def"))  , T("/abc/def/hi") );
	UNITST_EQ_TSTR( fname_getDir(buf, 100, T("abc.txt"))			 , T("") );
	UNITST_EQ_TSTR( fname_getDir(buf,	2, T("ab/c.txt"))			 , T("a") );
	UNITST_EQ_TSTR( fname_getDir(buf,	3, T("ab/c.txt"))			 , T("ab") );

	UNITST_EQ_TSTR( fname_getDrive(buf, 100, T("c:abcd/abc.def"))	   , T("c:") );
	UNITST_EQ_TSTR( fname_getDrive(buf, 100, T("/abc/def/hi/jk.def"))  , T("") );
	IF_W32 UNITST_EQ_TSTR( fname_getDrive(buf, 100, T("d:\\abc.txt"))		   , T("d:") );
	UNITST_EQ_TSTR( fname_getDrive(buf, 100, T("d:/abc.txt"))		   , T("d:") );
	UNITST_EQ_TSTR( fname_getDrive(buf, 100, T("http://homepage/abc.txt")), T("http:") );

	UNITST_EQ_TSTR( fname_getDriveRoot(buf, 100, T("c:abcd/abc.def"))	   , T("c:") );
	UNITST_EQ_TSTR( fname_getDriveRoot(buf, 100, T("c:/abcd/abc.def"))		, T("c:/") );
	IF_W32 UNITST_EQ_TSTR( fname_getDriveRoot(buf, 100, T("\\abc/def/hi/jk.def"))	, T("\\") );
	UNITST_EQ_TSTR( fname_getDriveRoot(buf, 100, T("/abc/def/hi/jk.def"))	, T("/") );
	UNITST_EQ_TSTR( fname_getDriveRoot(buf, 100, T("d:///c/abc.txt"))		  , T("d:///") );
	UNITST_EQ_TSTR( fname_getDriveRoot(buf, 100, T("http://homepage/abc.txt")), T("http://") );

	UNITST_EQ_TSTR( fname_delLastSep(fname_cpy(buf, 100, T("/base"))) , T("/base") );
	UNITST_EQ_TSTR( fname_delLastSep(fname_cpy(buf, 100, T("c:/base/"))), T("c:/base") );
	IF_W32 UNITST_EQ_TSTR( fname_delLastSep(fname_cpy(buf, 100, T("c:\\base\\"))), T("c:\\base") );
  #ifndef UNUSE_SJIS
	UNITST_EQ_TSTR( fname_delLastSep(fname_cpy(buf, 100, T("\"))), T("\") );
  #endif
	UNITST_EQ_TSTR( fname_delLastSep(fname_cpy(buf, 100, T("/"))), T("") );
	IF_W32 UNITST_EQ_TSTR( fname_delLastSep(fname_cpy(buf, 100, T("\\"))), T("") );

	UNITST_EQ_TSTR( fname_addSep(fname_cpy(buf, 100, T(""))		, 100), FNAME_SEP_STR );
	IF_W32 UNITST_EQ_TSTR( fname_addSep(fname_cpy(buf, 100, T("test"))	, 100), T("test\\") );
	else   UNITST_EQ_TSTR( fname_addSep(fname_cpy(buf, 100, T("test"))	, 100), T("test/") );
	UNITST_EQ_TSTR( fname_addSep(fname_cpy(buf, 100, T("test/")) , 100), T("test/") );
	IF_W32 UNITST_EQ_TSTR( fname_addSep(fname_cpy(buf, 100, T("test\\")), 100), T("test\\") );
 #ifndef UNUSE_SJIS
	UNITST_EQ_TSTR( fname_addSep(fname_cpy(buf, 100, T("\"))    , 100), T("\") FNAME_SEP_STR );
  #ifndef UNICODE
	UNITST_EQ_TSTR( fname_addSep(fname_cpy(buf, 100, T("\"))    ,   3), T("\") );
  #else
	UNITST_EQ_TSTR( fname_addSep(fname_cpy(buf, 100, T("\"))    ,   2), T("\") );
  #endif
 #endif

	UNITST_EQ_TSTR( fname_toUpper(fname_cpy(buf, 100, T("abc"))), T("ABC") );
	UNITST_EQ_TSTR( fname_toUpper(fname_cpy(buf, 100, T("Abc"))), T("ABC") );
	UNITST_EQ_TSTR( fname_toUpper(fname_cpy(buf, 100, T(""))), T("") );

	UNITST_EQ_TSTR( fname_toLower(fname_cpy(buf, 100, T("ABC"))), T("abc") );
	UNITST_EQ_TSTR( fname_toLower(fname_cpy(buf, 100, T("Abc"))), T("abc") );
	UNITST_EQ_TSTR( fname_toLower(fname_cpy(buf, 100, T("`ab"))), T("`ab") );

	UNITST_EQ_TSTR( fname_backslashToSlash(fname_cpy(buf, 100, T(""))), T("") );
	UNITST_EQ_TSTR( fname_backslashToSlash(fname_cpy(buf, 100, T("\\"))), T("/") );
  #ifndef UNUSE_SJIS
	UNITST_EQ_TSTR( fname_backslashToSlash(fname_cpy(buf, 100, T("\[X\\\"))), T("\[X/\") );
  #endif
	UNITST_EQ_TSTR( fname_backslashToSlash(fname_cpy(buf, 100, T("a/b\\c/d\\"))), T("a/b/c/d/") );

	UNITST_EQ_TSTR( fname_slashToBackslash(fname_cpy(buf, 100, T(""))), T("") );
	UNITST_EQ_TSTR( fname_slashToBackslash(fname_cpy(buf, 100, T("/"))), T("\\") );
  #ifndef UNUSE_SJIS
	UNITST_EQ_TSTR( fname_slashToBackslash(fname_cpy(buf, 100, T("\[X/\"))), T("\[X\\\") );
  #endif
	UNITST_EQ_TSTR( fname_slashToBackslash(fname_cpy(buf, 100, T("a/b\\c/d\\"))), T("a\\b\\c\\d\\") );

 #if 1
	const FNAME_CHAR* p = T("abc def \" spc \"   \"\"\"xyz\"\"\"\n");
	p = fname_scanArgStr(buf, 100, p);
	UNITST_EQ_TSTR( buf, T("abc") );
	p = fname_scanArgStr(buf, 100, p);
	UNITST_EQ_TSTR( buf, T("def") );
	p = fname_scanArgStr(buf, 100, p);
	UNITST_EQ_TSTR( buf, T(" spc ") );
	p = fname_scanArgStr(buf, 100, p);
	UNITST_EQ_TSTR( buf, T("\"xyz\"") );
	p = fname_scanArgStr(buf, 100, p);
	UNITST_EQ_TSTR( buf, T("") );
	UNITST_CHECK( p == 0);
  #endif

	UNITST_EQ_TSTR( fname_fullpath(buf, 100, T(".") 	 , T("/"))	   ,   FNAME_SEP_STR );
	UNITST_EQ_TSTR( fname_fullpathSL(buf, 100, T("./")	 , T("c:/bin")),   T("c:/bin/") );
	UNITST_EQ_TSTR( fname_fullpathSL(buf, 100, T("../")	 , T("c:/bin")),   T("c:/") );
	UNITST_EQ_TSTR( fname_fullpathSL(buf, 100, T("../test"), T("c:/bin")),	 T("c:/test") );
	UNITST_EQ_TSTR( fname_fullpathSL(buf, 100, T("../test/a/b/c/d/e/f/../../../g"), T("c:/bin")),	T("c:/test/a/b/c/g") );

	UNITST_EQ_TSTR( fname_fullpathSL(buf, 100, T("a:.")	   , T("/"))	 ,	 T("a:/") );
	UNITST_EQ_TSTR( fname_fullpathSL(buf, 100, T("a:./")	   , T("c:/bin")),	 T("c:/bin/") );
	UNITST_EQ_TSTR( fname_fullpathSL(buf, 100, T("a:../")	 , T("c:/bin")),	 T("c:/") );
	UNITST_EQ_TSTR( fname_fullpathSL(buf, 100, T("a:../test"), T("c:/bin")),	 T("c:/test") );
	UNITST_EQ_TSTR( fname_fullpathSL(buf, 100, T("a:../test/a/b/c/d/e/f/../../../g"), T("c:/bin")),	T("c:/test/a/b/c/g") );

	UNITST_EQ_TSTR( fname_fullpathSL(buf, 100, T("/.")	  , T("/")) 	,	T("/") );
	UNITST_EQ_TSTR( fname_fullpathSL(buf, 100, T("/./")	  , T("c:/bin/a/b")),	T("c:/") );
	UNITST_EQ_TSTR( fname_fullpathSL(buf, 100, T("/../")	  , T("c:/bin/a/b")),	T("c:/") );
	UNITST_EQ_TSTR( fname_fullpathSL(buf, 100, T("/../test"), T("c:/bin/a/b")),	T("c:/test") );
	UNITST_EQ_TSTR( fname_fullpathSL(buf, 100, T("/../test/a/b/c/d/e/f/../../../g"), T("c:/bin/c/d/")),	T("c:/test/a/b/c/g") );

	UNITST_EQ_TSTR( fname_fullpathSL(buf, 100, T("b:/.")		, T("/"))	  ,   T("b:/") );
	UNITST_EQ_TSTR( fname_fullpathSL(buf, 100, T("b:/./") 	, T("c:/bin/a/b")),   T("b:/") );
	UNITST_EQ_TSTR( fname_fullpathSL(buf, 100, T("b:/../")	, T("c:/bin/a/b")),   T("b:/") );
	UNITST_EQ_TSTR( fname_fullpathSL(buf, 100, T("b:/../test"), T("c:/bin/a/b")),	T("b:/test") );
	UNITST_EQ_TSTR( fname_fullpathSL(buf, 100, T("b:/../test/a/b/c/d/e/f/../../../g"), T("c:/bin/c/d/")),	T("b:/test/a/b/c/g") );

	IF_W32 UNITST_EQ_TSTR( fname_fullpathSL(buf, 100, T("//////te...st//1/./././././-/-/-/../..\\../2//.\\3.txt") , T("/")),	 T("///te...st/1/2/3.txt") );
	UNITST_EQ_TSTR( fname_fullpathSL(buf, 100, T("//////te...st//1/./././././-/-/-/../../../2//./3.txt") , T("/")),	 T("///te...st/1/2/3.txt") );

	IF_W32 UNITST_EQ_TSTR( fname_fullpathSL(buf, 100, T("./4/")	, T("//////te...st//1/./././././-/-/-/../../../2//.\\3"))  ,   T("///te...st/1/2/3/4/") );
	UNITST_EQ_TSTR( fname_fullpathSL(buf, 100, T("./4/")	, T("//////te...st//1/./././././-/-/-/../../../2//./3"))  ,   T("///te...st/1/2/3/4/") );

	IF_W32 UNITST_EQ_TSTR( fname_fullpathBS(buf, 100, T("a:../test\\a/b\\c/d\\e/f/../../../g"), T("c:/bin")),	T("c:\\test\\a\\b\\c\\g") );
	IF_W32 UNITST_EQ_TSTR( fname_fullpath(buf, 100, T("a:../test\\a/b\\c/d\\e/f/../../../g"), T("c:/bin")),	T("c:\\test\\a\\b\\c\\g") );
	else   UNITST_EQ_TSTR( fname_fullpath(buf, 100, T("a:../test/a/b/c/d/e/f/../../../g"), T("c:/bin")),	T("c:/test/a/b/c/g") );


	UNITST_EQ_TSTR( fname_relativePathSL(buf, 100, T(".") 	 , T("/"))	   ,   T("") );
	UNITST_EQ_TSTR( fname_relativePathSL(buf, 100, T("./")	 , T("c:/bin")),   T("") );
	UNITST_EQ_TSTR( fname_relativePathSL(buf, 100, T("../")	 , T("/bin")),	 T("../") );
	UNITST_EQ_TSTR( fname_relativePathSL(buf, 100, T("../test"), T("c:/bin")),	 T("../test") );
	UNITST_EQ_TSTR( fname_relativePathSL(buf, 100, T("../test/a/b/c/d/e/f/../../../g"), T("c:/bin")),	T("../test/a/b/c/g") );
	UNITST_EQ_TSTR( fname_relativePathSL(buf, 100, T("b:/bin") , T("c:/bin")),	 T("b:/bin") );
	UNITST_EQ_TSTR( fname_relativePathSL(buf, 100, T("c:/bin/a/b/c") , T("c:/bin")),	 T("a/b/c") );
	UNITST_EQ_TSTR( fname_relativePathSL(buf, 100, T("/bin/a/b/c/") , T("/bin")),	T("a/b/c/") );
	UNITST_EQ_TSTR( fname_relativePathSL(buf, 100, T("c:/bin/a/b/c.txt") , T("c:/bin/bin2/t")),	T("../../a/b/c.txt") );
	UNITST_EQ_TSTR( fname_relativePathSL(buf, 100, T("c:/bin/a/b/c/d/e/f/g/h/i/j/k.txt") , T("c:/bin/a/b/c/d/f/bin2/t")),	T("../../../e/f/g/h/i/j/k.txt") );

	IF_W32 UNITST_EQ_TSTR( fname_relativePathBS(buf, 100, T("c:/bin/a/b/c.txt") , T("c:/bin/bin2/t")),	T("..\\..\\a\\b\\c.txt") );
	IF_W32 UNITST_EQ_TSTR( fname_relativePath(buf, 100, T("c:/bin/a/b/c.txt") , T("c:/bin/bin2/t")),	T("..\\..\\a\\b\\c.txt") );
	else   UNITST_EQ_TSTR( fname_relativePath(buf, 100, T("c:/bin/a/b/c.txt") , T("c:/bin/bin2/t")),	T("../../a/b/c.txt") );

	UNITST_CHECK( fname_matchWildCard(T("abc"), T("abc") ) != 0 );
	UNITST_CHECK( fname_matchWildCard(T("abc"), T("abcd")) == 0 );
	UNITST_CHECK( fname_matchWildCard(T("abc?"), T("abcd")) != 0 );
	UNITST_CHECK( fname_matchWildCard(T("a?cd"), T("abcd")) != 0 );
	IF_W32 UNITST_CHECK( fname_matchWildCard(T("a?d"), T("ad")) != 0 );
	UNITST_CHECK( fname_matchWildCard(T("abc*"), T("abcd")) != 0 );
	UNITST_CHECK( fname_matchWildCard(T("abc*"), T("abc")) != 0 );
	UNITST_CHECK( fname_matchWildCard(T("a*d"), T("abcd")) != 0 );
	UNITST_CHECK( fname_matchWildCard(T("a*d"), T("aabd")) != 0 );

  #ifndef UNUSE_SJIS
	UNITST_CHECK( fname_matchWildCard(T("a*d"), T("a\\d")) != 0 );
  #endif
	UNITST_CHECK( fname_matchWildCard(T("a*d"), T("abc/d")) == 0 );
	IF_W32 UNITST_CHECK( fname_matchWildCard(T("a*d"), T("a\\bcd")) == 0 );
	UNITST_CHECK( fname_matchWildCard(T("a*d"), T("a/bcd")) == 0 );
	UNITST_CHECK( fname_matchWildCard(T("a**d"), T("ab/cd")) != 0 );
	UNITST_CHECK( fname_matchWildCard(T("**/test.*"), T("ab/cd/test.log")) != 0 );
  #ifndef UNUSE_SJIS
	UNITST_CHECK( fname_matchWildCard(T("**/test.*"), T("ab/cd/\/test.")) != 0 );
  #endif
	UNITST_CHECK( fname_matchWildCard(T("**/test.*"), T("ab/cd/test")) == 0 );
	UNITST_CHECK( fname_matchWildCard(T("**/test.*"), T("test.log")) == 0 );

}

