/**
 *	@file	test_fname_string.cpp
 *	@brief
 *	@author	tenk*
 *	@date	2010-??
 *	@note
 *		fname_string.h ̃eXg.
 */

#define USE_UNITST
#define USE_UNITST_STRSTREAM

#include <stdio.h>
#include <stdlib.h>
#include "unitst.h"
#include <string>
#include <vector>

#if 1
#ifdef _WIN32
#include <windows.h>
#endif
#include "../fname_string.h"
#ifdef UNICODE
#define STRING		std::wstring
#else
#define STRING		std::string
#endif
#else
#include <windows.h>
#include <atlbase.h>
#include <atlapp.h>
#include <atlmisc.h>
#include "../fname_cstring.h"
#define STRING		CString
#endif


// ===========================================================================
#ifdef UNICODE
#define T(x)		L##x
#else
#define T(x)		x
#endif

#if defined _WIN32 || defined _DOS || defined _MSDOS
#define IF_W32	if (1)
#else
#define IF_W32	if (0)
#endif


#if !(defined __DMC__)

UNITST_TEST(fname_string) {
	const STRING	test1 = T("C:/Dummy/File.txt");
	const STRING	test2 = T("c:/Dummy/FiLE.txt");

	UNITST_CHECK( fname_cmp(test1, T("d:")) < 0 );
	UNITST_CHECK( fname_cmp(test1, T("B:")) > 0 );
	UNITST_CHECK( fname_cmp(T("C:/Dummy/File.txt"), STRING(T("B:"))) > 0 );
	IF_W32 UNITST_CHECK( fname_cmp(test1, test2) == 0 );
	else   UNITST_CHECK( fname_cmp(test1, test2) != 0 );

	UNITST_CHECK( fname_n_cmp(test2, T("d:"), 2) < 0 );
	UNITST_CHECK( fname_n_cmp(test1, T("B:/"), 3) > 0 );
	UNITST_CHECK( fname_n_cmp(test1, T("C:/De"), 4) == 0 );
	UNITST_CHECK( fname_n_cmp(T("C:/Dummy/File.txt"), STRING(T("C:/De")), 5) > 0 );
	IF_W32 UNITST_CHECK( fname_n_cmp(test1, test2, 0xffff) == 0 );
	else   UNITST_CHECK( fname_n_cmp(test1, test2, 0xffff) != 0 );

	UNITST_CHECK( fname_equLong(test1, T("C:/Dummy")) != 0 );
	UNITST_CHECK( fname_equLong(test1, T("C:/Bs")) == 0 );

	UNITST_CHECK( fname_isAbs(STRING(T("c:/test1.txt"))) != 0);
	UNITST_CHECK( fname_isAbs(STRING(T("/test1.txt"))) != 0);
	UNITST_CHECK( fname_isAbs(STRING(T("c:test1.txt"))) == 0 );
	UNITST_CHECK( fname_isAbs(STRING(T("test1.txt"))) == 0 );

	UNITST_CHECK( fname_haveDrive(STRING(T("c:test1.txt"))) != 0);
	UNITST_CHECK( fname_haveDrive(STRING(T("test1.txt"))) == 0 );

	UNITST_EQ( fname_adjust_size(STRING(T("c:/test.c")), 20), 9 );
	UNITST_EQ( fname_adjust_size(STRING(T("c:/test.c")), 10), 9 );
	UNITST_EQ( fname_adjust_size(STRING(T("c:/test.c")),  9), 8 );
  #if defined _WIN32 && !(defined UNICODE)
	UNITST_EQ( fname_adjust_size(STRING(T("c:eXg.c")), 8), 6 );
  #endif

	UNITST_EQ( fname_baseName(STRING(T("abc.text"))),			  T("abc.text") );
	UNITST_EQ( fname_baseName(STRING(T("a:abc.text"))),		   	  T("abc.text") );
	UNITST_EQ( fname_baseName(STRING(T("c:/abc.text"))),		  T("abc.text") );
	UNITST_EQ( fname_baseName(STRING(T("c:/basesss/abc.text"))), T("abc.text") );
	UNITST_EQ( fname_baseName(STRING(T("/basesss/abc.text"))),	  T("abc.text") );

	UNITST_EQ( fname_ext(STRING(T("/basesss/abc.ext"))),	 T("ext") );
	UNITST_EQ( fname_ext(STRING(T("/basesss/abc")	 )),	 T("") );
	UNITST_EQ( fname_ext(STRING(T("/basesss/abc.")	 )),	 T("") );

	UNITST_EQ( fname_skipDrive(STRING(T("abc.text"))),			  T("abc.text") );
	UNITST_EQ( fname_skipDrive(STRING(T("a:abc.text"))), 		  T("abc.text") );
	UNITST_EQ( fname_skipDrive(STRING(T("c:/abc.text"))),		  T("/abc.text") );
	UNITST_EQ( fname_skipDrive(STRING(T("c:/basesss/abc.text"))), T("/basesss/abc.text") );
	UNITST_EQ( fname_skipDrive(STRING(T("/basesss/abc.text"))),	  T("/basesss/abc.text") );

	UNITST_EQ( fname_skipDriveRoot(STRING(T("abc.text"))),			  T("abc.text") );
	UNITST_EQ( fname_skipDriveRoot(STRING(T("a:abc.text"))), 		  T("abc.text") );
	UNITST_EQ( fname_skipDriveRoot(STRING(T("c:/abc.text"))),		  T("abc.text") );
	UNITST_EQ( fname_skipDriveRoot(STRING(T("c:/basesss/abc.text"))), T("basesss/abc.text") );
	UNITST_EQ( fname_skipDriveRoot(STRING(T("/basesss/abc.text"))),	  T("basesss/abc.text") );

	UNITST_EQ( fname_getNoExt(STRING(T("c:abcd/abc.defghijk"))), T("c:abcd/abc") );
	UNITST_EQ( fname_getNoExt(STRING(T("abc.def")			 )), T("abc") );
	UNITST_EQ( fname_getNoExt(STRING(T("abc.")				 )), T("abc") );
	UNITST_EQ( fname_getNoExt(STRING(T("abc")				 )), T("abc") );

	UNITST_EQ( fname_getBaseNameNoExt(STRING(T("c:abcd/abc.defghijk"))), T("abc") );
	UNITST_EQ( fname_getBaseNameNoExt(STRING(T("c:dir/abc.")		 )), T("abc") );
	UNITST_EQ( fname_getBaseNameNoExt(STRING(T("c:dir/abc") 		 )), T("abc") );

	UNITST_EQ( fname_setExt(STRING(T("c:abcd/abc.def")), T("txt")), T("c:abcd/abc.txt") );
	UNITST_EQ( fname_setExt(T("c:abcd/abc."	 )		   , STRING(T("txt"))), T("c:abcd/abc.txt") );
	UNITST_EQ( fname_setExt(STRING(T("c:abcd/abc"	 )), STRING(T("txt"))), T("c:abcd/abc.txt") );
	UNITST_EQ( fname_setExt(STRING(T("c:abcd/abc.def")), T(""))	 , T("c:abcd/abc") );
	UNITST_EQ( fname_setExt(STRING(T("c:abcd/abc."	 )), T(""))	 , T("c:abcd/abc") );
	UNITST_EQ( fname_setExt(STRING(T("c:abcd/abc"	 )), T(""))	 , T("c:abcd/abc") );

	UNITST_EQ( fname_setDefaultExt(STRING(T("c:abcd/abc.def")), T("txt")), T("c:abcd/abc.def") );
	UNITST_EQ( fname_setDefaultExt( 	 (T("c:abcd/abc."	)), STRING(T("txt"))), T("c:abcd/abc.txt") );
	UNITST_EQ( fname_setDefaultExt(STRING(T("c:abcd/abc"	)), STRING(T("txt"))), T("c:abcd/abc.txt") );
	UNITST_EQ( fname_setDefaultExt(STRING(T("c:abcd/abc.def")), T(""))	, T("c:abcd/abc.def") );
	UNITST_EQ( fname_setDefaultExt(STRING(T("c:abcd/abc."	)), T(""))	, T("c:abcd/abc") );
	UNITST_EQ( fname_setDefaultExt(STRING(T("c:abcd/abc"	)), T(""))	, T("c:abcd/abc") );

	UNITST_CHECK( fname_checkLastSep( STRING(T("abx")) ) == 0 );
  #ifndef UNUSE_SJIS
	UNITST_CHECK( fname_checkLastSep( STRING(T("\")) ) == 0 );
  #endif
	UNITST_CHECK( fname_checkLastSep( STRING(T("")) ) == 0 );
	UNITST_EQ( fname_checkLastSep( STRING(T("abx/")) ), STRING(T("/")) );
	UNITST_EQ( fname_checkLastSep( STRING(T("abx/")) ), STRING(T("/")) );
	UNITST_EQ( fname_checkLastSep( STRING(T("/") )), STRING(T("/")) );
	UNITST_EQ( fname_checkLastSep( STRING(T("/") )), STRING(T("/")) );

	IF_W32 UNITST_EQ( fname_join(STRING(T("c:abcd/def")), T("test.txt")), T("c:abcd/def\\test.txt") );
	else   UNITST_EQ( fname_join(STRING(T("c:abcd/def")), T("test.txt")), T("c:abcd/def/test.txt") );
	IF_W32 UNITST_EQ( fname_join((T("c:abcd/def")), STRING(T("ghi/test.txt"))), T("c:abcd/def\\ghi/test.txt") );
	else   UNITST_EQ( fname_join((T("c:abcd/def")), STRING(T("ghi/test.txt"))), T("c:abcd/def/ghi/test.txt") );

	UNITST_EQ( fname_join(STRING(T("c:abcd/def/")), T("test.txt")), T("c:abcd/def/test.txt") );
	UNITST_EQ( fname_join(STRING(T("c:abcd/def/")), STRING(T("ghi/test.txt"))), T("c:abcd/def/ghi/test.txt") );

	UNITST_EQ( fname_getDir(STRING(T("c:abcd/abc.def"))) 	 , T("c:abcd") );
	UNITST_EQ( fname_getDir(STRING(T("/abc/def/hi/jk.def")))  , T("/abc/def/hi") );
	UNITST_EQ( fname_getDir(STRING(T("abc.txt")))			 , T("") );

	UNITST_EQ( fname_getDrive(STRING(T("c:abcd/abc.def")))	   , T("c:") );
	UNITST_EQ( fname_getDrive(STRING(T("/abc/def/hi/jk.def")))	, T("") );
	UNITST_EQ( fname_getDrive(STRING(T("d:/abc.txt")))		   , T("d:") );
	UNITST_EQ( fname_getDrive(STRING(T("http://homepage/abc.txt"))), T("http:") );

	UNITST_EQ( fname_getDriveRoot(STRING(T("c:abcd/abc.def")))	   , T("c:") );
	UNITST_EQ( fname_getDriveRoot(STRING(T("c:/abcd/abc.def")))		, T("c:/") );
	UNITST_EQ( fname_getDriveRoot(STRING(T("/abc/def/hi/jk.def")))	, T("/") );
	UNITST_EQ( fname_getDriveRoot(STRING(T("d:///c/abc.txt")))		  , T("d:///") );
	UNITST_EQ( fname_getDriveRoot(STRING(T("http://homepage/abc.txt"))), T("http://") );

	UNITST_EQ( fname_delLastSep((STRING(T("/base")))) , T("/base") );
	UNITST_EQ( fname_delLastSep((STRING(T("c:/base/")))), T("c:/base") );
	UNITST_EQ( fname_delLastSep((STRING(T("c:/base/")))), T("c:/base") );
  #ifndef UNUSE_SJIS
	UNITST_EQ( fname_delLastSep((STRING(T("\")))), T("\") );
  #endif
	UNITST_EQ( fname_delLastSep((STRING(T("/")))), T("") );
	IF_W32 UNITST_EQ( fname_delLastSep((STRING(T("\\")))), T("") );

	UNITST_EQ( fname_addSep((STRING(T("")))		), FNAME_SEP_STR );
	IF_W32 UNITST_EQ( fname_addSep((STRING(T("test")))	), T("test\\") );
	else   UNITST_EQ( fname_addSep((STRING(T("test")))	), T("test/") );
	UNITST_EQ( fname_addSep((STRING(T("test/"))) ), T("test/") );
	IF_W32 UNITST_EQ( fname_addSep((STRING(T("test\\")))), T("test\\") );
  #ifndef UNUSE_SJIS
	UNITST_EQ( fname_addSep((STRING(T("\")))    ), T("\") FNAME_SEP_STR );
  #endif

	UNITST_EQ( fname_toUpper((STRING(T("abc")))), T("ABC") );
	UNITST_EQ( fname_toUpper((STRING(T("Abc")))), T("ABC") );
	UNITST_EQ( fname_toUpper((STRING(T("")))), T("") );

	UNITST_EQ( fname_toLower((STRING(T("ABC")))), T("abc") );
	UNITST_EQ( fname_toLower((STRING(T("Abc")))), T("abc") );
	UNITST_EQ( fname_toLower((STRING(T("`ab")))), T("`ab") );

	UNITST_EQ( fname_backslashToSlash((STRING(T("")))), T("") );
	UNITST_EQ( fname_backslashToSlash((STRING(T("\\")))), T("/") );
	UNITST_EQ( fname_backslashToSlash((STRING(T("[\\Ђ傤")))), T("[/Ђ傤") );
  #ifndef UNUSE_SJIS
	UNITST_EQ( fname_backslashToSlash((STRING(T("\[X\\\")))), T("\[X/\") );
  #endif
	UNITST_EQ( fname_backslashToSlash((STRING(T("a/b\\c/d\\")))), T("a/b/c/d/") );

	UNITST_EQ( fname_slashToBackslash((STRING(T("")))), T("") );
	UNITST_EQ( fname_slashToBackslash((STRING(T("/")))), T("\\") );
	UNITST_EQ( fname_slashToBackslash((STRING(T("[/Ђ傤")))), T("[\\Ђ傤") );
  #ifndef UNUSE_SJIS
	UNITST_EQ( fname_slashToBackslash((STRING(T("\[X/\")))), T("\[X\\\") );
  #endif
	UNITST_EQ( fname_slashToBackslash((STRING(T("a/b\\c/d\\")))), T("a\\b\\c\\d\\") );

	UNITST_EQ( fname_fullpath(STRING(T(".")) 	 , T("/"))	   ,   FNAME_SEP_STR );
	UNITST_EQ( fname_fullpathSL(STRING(T("./"))	 , STRING(T("c:/bin"))),   T("c:/bin/") );
	UNITST_EQ( fname_fullpathSL(	  (T("../"))	 , STRING(T("c:/bin"))),   T("c:/") );
	UNITST_EQ( fname_fullpathSL(STRING(T("../test")), T("c:/bin")),   T("c:/test") );
	UNITST_EQ( fname_fullpathSL(STRING(T("../test/a/b/c/d/e/f/../../../g")), T("c:/bin")),	 T("c:/test/a/b/c/g") );

	UNITST_EQ( fname_fullpathSL(STRING(T("a:."))	   , T("/"))	 ,	 T("a:/") );
	UNITST_EQ( fname_fullpathSL(STRING(T("a:./"))	   , T("c:/bin")),	 T("c:/bin/") );
	UNITST_EQ( fname_fullpathSL(STRING(T("a:../"))	  , T("c:/bin")),	 T("c:/") );
	UNITST_EQ( fname_fullpathSL(STRING(T("a:../test")), T("c:/bin")),	 T("c:/test") );
	UNITST_EQ( fname_fullpathSL(STRING(T("a:../test/a/b/c/d/e/f/../../../g")), T("c:/bin")),	T("c:/test/a/b/c/g") );

	UNITST_EQ( fname_fullpathSL(STRING(T("/."))	  , T("/")) 	,	T("/") );
	UNITST_EQ( fname_fullpathSL(STRING(T("/./"))	  , T("c:/bin/a/b")),	T("c:/") );
	UNITST_EQ( fname_fullpathSL(STRING(T("/../"))	  , T("c:/bin/a/b")),	T("c:/") );
	UNITST_EQ( fname_fullpathSL(STRING(T("/../test")), T("c:/bin/a/b")),	T("c:/test") );
	UNITST_EQ( fname_fullpathSL(STRING(T("/../test/a/b/c/d/e/f/../../../g")), T("c:/bin/c/d/")),	T("c:/test/a/b/c/g") );

	UNITST_EQ( fname_fullpathSL(STRING(T("b:/."))		, T("/"))	  ,   T("b:/") );
	UNITST_EQ( fname_fullpathSL(STRING(T("b:/./")) 	, T("c:/bin/a/b")),   T("b:/") );
	UNITST_EQ( fname_fullpathSL(STRING(T("b:/../"))	, T("c:/bin/a/b")),   T("b:/") );
	UNITST_EQ( fname_fullpathSL(STRING(T("b:/../test")), T("c:/bin/a/b")),	 T("b:/test") );
	UNITST_EQ( fname_fullpathSL(STRING(T("b:/../test/a/b/c/d/e/f/../../../g")), T("c:/bin/c/d/")),	 T("b:/test/a/b/c/g") );

	UNITST_EQ( fname_fullpathSL(STRING(T("//////te...st//1/./././././-/-/-/../../../2//./3.txt")) , T("/")),	 T("///te...st/1/2/3.txt") );

	UNITST_EQ( fname_fullpathSL(STRING(T("./4/"))	, T("//////te...st//1/./././././-/-/-/../../../2//./3"))  ,   T("///te...st/1/2/3/4/") );

	IF_W32 UNITST_EQ( fname_fullpath(STRING(T("a:../test/a/b/c/d/e/f/../../../g")), T("c:/bin")),	T("c:\\test\\a\\b\\c\\g") );
	else   UNITST_EQ( fname_fullpath(STRING(T("a:../test/a/b/c/d/e/f/../../../g")), T("c:/bin")),	T("c:/test/a/b/c/g") );
	IF_W32 UNITST_EQ( fname_fullpath(		(T("a:../test/a/b/c/d/e/f/../../../g")), STRING(T("c:/bin"))),	T("c:\\test\\a\\b\\c\\g") );
	else   UNITST_EQ( fname_fullpath(		(T("a:../test/a/b/c/d/e/f/../../../g")), STRING(T("c:/bin"))),	T("c:/test/a/b/c/g") );
	IF_W32 UNITST_EQ( fname_fullpath(STRING(T("a:../test/a/b/c/d/e/f/../../../g")), STRING(T("c:/bin"))),	T("c:\\test\\a\\b\\c\\g") );
	else   UNITST_EQ( fname_fullpath(STRING(T("a:../test/a/b/c/d/e/f/../../../g")), STRING(T("c:/bin"))),	T("c:/test/a/b/c/g") );


	UNITST_EQ( fname_relativePathSL(STRING(T(".")) 	 , T("/"))	   ,   T("") );
	UNITST_EQ( fname_relativePathSL(STRING(T("./"))	 , STRING(T("c:/bin"))),   T("") );
	UNITST_EQ( fname_relativePathSL(	  (T("../")) , STRING(T("c:/bin"))),   T("../") );
	UNITST_EQ( fname_relativePathSL(STRING(T("../test")), T("c:/bin")),   T("../test") );
	UNITST_EQ( fname_relativePathSL(STRING(T("../test/a/b/c/d/e/f/../../../g")), T("c:/bin")),	 T("../test/a/b/c/g") );
	UNITST_EQ( fname_relativePathSL(STRING(T("b:/bin")) , T("c:/bin")),   T("b:/bin") );
	UNITST_EQ( fname_relativePathSL(STRING(T("c:/bin/a/b/c")) , T("c:/bin")),	 T("a/b/c") );
	UNITST_EQ( fname_relativePathSL(STRING(T("c:/bin/a/b/c/")) , T("c:/bin")),	 T("a/b/c/") );
	UNITST_EQ( fname_relativePathSL(STRING(T("c:/bin/a/b/c.txt")) , T("c:/bin/bin2/t")),	T("../../a/b/c.txt") );
	UNITST_EQ( fname_relativePathSL(STRING(T("c:/bin/a/b/c/d/e/f/g/h/i/j/k.txt")) , T("c:/bin/a/b/c/d/f/bin2/t")),	T("../../../e/f/g/h/i/j/k.txt") );

	IF_W32 UNITST_EQ( fname_relativePath(STRING(T("c:/bin/a/b/c.txt")) , T("c:/bin/bin2/t")),	T("..\\..\\a\\b\\c.txt") );
	else   UNITST_EQ( fname_relativePath(STRING(T("c:/bin/a/b/c.txt")) , T("c:/bin/bin2/t")),	T("../../a/b/c.txt") );
	IF_W32 UNITST_EQ( fname_relativePath(		(T("c:/bin/a/b/c.txt")) , STRING(T("c:/bin/bin2/t"))),	T("..\\..\\a\\b\\c.txt") );
	else   UNITST_EQ( fname_relativePath(		(T("c:/bin/a/b/c.txt")) , STRING(T("c:/bin/bin2/t"))),	T("../../a/b/c.txt") );
	IF_W32 UNITST_EQ( fname_relativePath(STRING(T("c:/bin/a/b/c.txt")) , STRING(T("c:/bin/bin2/t"))),	T("..\\..\\a\\b\\c.txt") );
	else   UNITST_EQ( fname_relativePath(STRING(T("c:/bin/a/b/c.txt")) , STRING(T("c:/bin/bin2/t"))),	T("../../a/b/c.txt") );

	UNITST_CHECK( fname_matchWildCard(STRING(T("abc")), T("abc") ) != 0 );
	UNITST_CHECK( fname_matchWildCard(T("abc"), STRING(T("abcd"))) == 0 );
	UNITST_CHECK( fname_matchWildCard(STRING(T("abc?")), STRING(T("abcd"))) != 0 );

	std::vector<STRING> vec;
	fname_scanArgStr(vec, T("abc def \" spc \"   \"\"\"xyz\"\"\"\n"));
	UNITST_EQ(vec.size(), 4);
	if (vec.size() > 0) UNITST_EQ( vec[0], T("abc") );
	if (vec.size() > 1) UNITST_EQ( vec[1], T("def") );
	if (vec.size() > 2) UNITST_EQ( vec[2], T(" spc ") );
	if (vec.size() > 3) UNITST_EQ( vec[3], T("\"xyz\"") );
	vec.clear();
	fname_scanArgStr(vec, T("c:\\;c:/bin;c:\\program files\\abc;\"c:\\hoge hoge\""), T(';'));
	UNITST_EQ(vec.size(), 4);
	if (vec.size() > 0) UNITST_EQ( vec[0], T("c:\\") );
	if (vec.size() > 1) UNITST_EQ( vec[1], T("c:/bin") );
	if (vec.size() > 2) UNITST_EQ( vec[2], T("c:\\program files\\abc") );
	if (vec.size() > 3) UNITST_EQ( vec[3], T("c:\\hoge hoge") );
}

#endif
